==> ask-cli-1.5.0.tgz: package/lib/utils/hook-manager.js <==
const fs = require('fs');
const path = require('path');
const shell = require('shelljs');
const mkdirp = require('mkdirp');
const url = require('url');
const jsonfile = require('jsonfile');
const async = require('async');

const httpClient = require('@src/clients/http-client');
const CONSTANTS = require('@src/utils/constants');
const jsonUtility = require('@src/utils/json-utility');
const displayUtil = require('@src/utils/display-utility');


class HookManager {
    /**
     * @param {string} category  type of the hook
     * @param {string} skillName  the skill name
     * @param {string} runTime    the lambda run time
     * @param {boolean} doDebug   the ASK CLI debug mode
     * @param {string} target     the target for deploy
     */
    constructor(category, skillName, runTime, profile, doDebug, target) {
        this.category = category;
        this.skillName = skillName;
        this.doDebug = doDebug === true;
        this.osType = process.platform === 'win32' ? 'WINDOWS' : 'LINUX';
        this.target = target;
        this.profile = profile;
        /* TODO : this.hookPath value to be consistent for all hooks executions */
        if (this.category === CONSTANTS.HOOKS.CATEGORIES.POST_NEW || this.category === CONSTANTS.HOOKS.CATEGORIES.POST_CLONE) {
            this.hookPath = path.join(process.cwd(), skillName, 'hooks');
            this.scriptPath = path.join(this.hookPath, CONSTANTS.HOOKS.CATEGORIES.FILE_NAME.POST_NEW[this.osType]);
        } else if (this.category === CONSTANTS.HOOKS.CATEGORIES.PRE_DEPLOY) {
            this.hookPath = path.join(process.cwd(), 'hooks');
            this.scriptPath = path.join(this.hookPath, CONSTANTS.HOOKS.CATEGORIES.FILE_NAME.PRE_DEPLOY[this.osType]);
        }
        this.runTime = runTime || this._getRunTime();
    }

    /**
     * Validates if the skill directory and skillName are consistent and if hook folder exists
     * @param {function} callback
     */
    validate(callback) {
        if (this.category === CONSTANTS.HOOKS.CATEGORIES.POST_NEW || this.category === CONSTANTS.HOOKS.CATEGORIES.POST_CLONE) {
            callback(null, fs.existsSync(this.hookPath));
        } else if (this.category === CONSTANTS.HOOKS.CATEGORIES.PRE_DEPLOY) {
            if (!this._checkDirConsistency()) {
                callback(`[Error]: Directory name is not consistent with the skill name present in skill.json: ${this.skillName}. Please change your project directory name accordingly.`, null);
            } else {
                callback(null, fs.existsSync(this.hookPath));
            }
        }
    }

    /**
     * Create hook folder and download corresponding language specific hook scripts
     * @param {function} callback
     */
    createHook(callback) {
        try {
            console.info('[Info]: Could not find hooks folder, creating a new hooks folder and downloading scripts.');
            mkdirp.sync(this.hookPath);
        } catch (err) {
            console.error(`[Error]: Cannot create directory path for: ${this.hookPath}`);
            callback(err);
            return;
        }
        /* To support hook logic for existing node skills (backward compatibility) */
        if (this.runTime && this.runTime.startsWith('node')) {
            let scriptList = null;
            if (this.category === CONSTANTS.HOOKS.CATEGORIES.POST_NEW) {
                scriptList = [CONSTANTS.HOOKS.CATEGORIES.FILE_NAME.POST_NEW[this.osType], CONSTANTS.HOOKS.CATEGORIES.FILE_NAME.PRE_DEPLOY[this.osType]];
            } else {
                scriptList = [CONSTANTS.HOOKS.CATEGORIES.FILE_NAME.PRE_DEPLOY[this.osType]];
            }
            const scriptPath = url.resolve(CONSTANTS.HOOKS.TEMPLATES_URL, 'node/');
            this._cloneHookScripts(scriptPath, scriptList, (err) => {
                callback(err);
            });
        } else {
        // Creating an empty pre_deploy_hook for all skills which are not node runtime.
            const hookScript = path.join(this.hookPath, CONSTANTS.HOOKS.CATEGORIES.FILE_NAME.PRE_DEPLOY[this.osType]);
            fs.writeFileSync(hookScript, '', { mode: 0o755 });
            callback(null);
        }
    }

    /**
     * Executes the hook scripts
     * @param {function} callback
     */
    execute(callback) {
        let command;
        if (this.osType === 'WINDOWS') {
            const doDebug = this.doDebug ? '$True' : '$False';
            command = `PowerShell.exe -Command "& {& '${this.scriptPath}' ${this.skillName} ${doDebug} ${this.target}}"`;
        } else {
            command = `${this.scriptPath} ${this.skillName} ${this.doDebug} ${this.target}`;
        }
        shell.exec(command, { async: true }, (code, stdout, stderr) => {
            if (code !== 0) {
                callback('[Error]: Hook Scripts failed', null);
            } else if (stderr) {
                callback('[Error]: Hook Scripts failed', null);
            } else {
                callback(null, `Successful executed ${this.category} hook script`);
            }
        });
    }

    /**
     * Check if the directory created for skill and the user input skillName are equal
     */
    _checkDirConsistency() {
        return this.skillName === path.basename(process.cwd());
    }

    /**
     * Returns a source directory path if exists from the skill.json
     */
    _getSourceDir() {
        const skillFilePath = path.join(this.hookPath, '../skill.json');
        const manifest = jsonfile.readFileSync(skillFilePath);
        const apiList = jsonUtility.getPropertyValueFromObject(manifest, ['manifest', 'apis']);
        if (apiList) {
            for (const domain of Object.keys(apiList)) {
                const endpoint = apiList[domain].endpoint;
                if (endpoint && endpoint.sourceDir) {
                    return endpoint.sourceDir;
                }
            }
        }
        return null;
    }

    /**
     * Extract runtime from ask/config file and also check for multiple language runtime
     */
    _getRunTime() {
        let runtime = null;
        const configPath = path.join(this.hookPath, '..', '.ask', 'config');
        const config = jsonUtility.read(configPath);
        const defaultConfig = jsonUtility.getPropertyValueFromObject(config, ['deploy_settings', this.profile, 'resources', 'lambda']);
        if (defaultConfig) {
            for (const lambda of defaultConfig) {
                if (!runtime && lambda.runtime) {
                    runtime = lambda.runtime.replace(/[^a-z]/gi, '');
                } else if (lambda.runtime && !lambda.runtime.startsWith(runtime)) {
                    console.warn('[Warn]: Different lambda runtime found. CLI currently does not support this functionality.'
                    + 'Developers should either work on their own hook scripts to build lambda for different language. '
                    + 'Or use only one languages for the skill.\n');
                    return null;
                }
            }
        }
        if (!runtime) {
            // check if package.json exists in template
            if (!this._getSourceDir()) {
                return null;
            }
            const packagePath = path.join(this.hookPath, '..', this._getSourceDir(), 'package.json');
            return fs.existsSync(packagePath) ? 'node' : 'python';
        }
        return runtime;
    }

    /**
     * Download hook scripts from the given url
     * @param {string} scriptUrl the url for get request
     * @param {function} callback
     */
    _getHookScripts(scriptUrl, callback) {
        const params = {
            url: scriptUrl,
            method: 'GET'
        };
        httpClient.request(params, 'getHookScripts', this.doDebug, (error, response) => {
            if (error || response.statusCode === null) {
                callback(error, null);
            } else {
                callback(null, response.body);
            }
        });
    }

    /**
     * Download hook scripts from S3 bucket to provide hook functionality for existing skills
     * @param {string} scriptPath the url for scripts
     * @param {list} scriptList   the list of scripts to download
     * @param {function} callback
     */
    _cloneHookScripts(scriptPath, scriptList, callback) {
        const installSpinner = new displayUtil.Spinner('Downloading the hooks scripts');
        installSpinner.start();
        async.forEach(scriptList, (script, getScriptCallback) => {
            const cloneDirLeaf = path.join(this.hookPath, script);
            const scriptUrl = url.resolve(scriptPath, script);
            this._getHookScripts(scriptUrl, (error, data) => {
                if (error) {
                    getScriptCallback(error);
                    return;
                }
                if (data) {
                    fs.writeFileSync(cloneDirLeaf, data, { mode: 0o755 });
                    getScriptCallback();
                }
            });
        }, (error) => {
            installSpinner.stop();
            callback(error);
        });
        installSpinner.stop();
    }
}

/**
 * Instantiate the hook object after successful validation or throw error.
 * @param {string} category   type of the hook
 * @param {string} skillName  the skill name
 * @param {string} runTime    the lambda run time
 * @param {boolean} doDebug   the ASK CLI debug mode
 * @param {string} target     the target for deploy
 */
function createManager(category, skillName, runTime, profile, doDebug, target, callback) {
    // instantiate
    const hookManager = new HookManager(category, skillName, runTime, profile, doDebug, target);
    // deep validation
    hookManager.validate((validateError, hookExists) => {
        if (validateError) {
            callback(validateError, null);
            return;
        }
        if (!hookExists) {
            hookManager.createHook((err) => {
                callback(err, err ? null : hookManager);
            });
        } else {
            callback(null, hookManager);
        }
    });
}

module.exports = {
    HookManager,
    createManager
};

==> ask-cli-1.6.0.tgz: package/lib/utils/hook-manager.js <==
const fs = require('fs');
const path = require('path');
const shell = require('shelljs');
const mkdirp = require('mkdirp');
const url = require('url');
const jsonfile = require('jsonfile');
const async = require('async');

const httpClient = require('@src/clients/http-client');
const CONSTANTS = require('@src/utils/constants');
const jsonUtility = require('@src/utils/json-utility');
const displayUtil = require('@src/utils/display-utility');


class HookManager {
    /**
     * @param {string} category  type of the hook
     * @param {string} skillName  the skill name
     * @param {string} runTime    the lambda run time
     * @param {boolean} doDebug   the ASK CLI debug mode
     * @param {string} target     the target for deploy
     */
    constructor(category, skillName, runTime, profile, doDebug, target) {
        this.category = category;
        this.skillName = skillName;
        this.doDebug = doDebug === true;
        this.osType = process.platform === 'win32' ? 'WINDOWS' : 'LINUX';
        this.target = target;
        this.profile = profile;
        /* TODO : this.hookPath value to be consistent for all hooks executions */
        if (this.category === CONSTANTS.HOOKS.CATEGORIES.POST_NEW || this.category === CONSTANTS.HOOKS.CATEGORIES.POST_CLONE) {
            this.hookPath = path.join(skillName, 'hooks');
            this.scriptPath = path.join(this.hookPath, CONSTANTS.HOOKS.CATEGORIES.FILE_NAME.POST_NEW[this.osType]);
        } else if (this.category === CONSTANTS.HOOKS.CATEGORIES.PRE_DEPLOY) {
            this.hookPath = 'hooks';
            this.scriptPath = path.join(this.hookPath, CONSTANTS.HOOKS.CATEGORIES.FILE_NAME.PRE_DEPLOY[this.osType]);
        }
        this.runTime = runTime || this._getRunTime();
    }

    /**
     * Create hook folder and download corresponding language specific hook scripts
     * @param {function} callback
     */
    createHook(callback) {
        try {
            console.log('[Info]: Could not find hooks folder, creating a new hooks folder and downloading scripts.');
            mkdirp.sync(this.hookPath);
        } catch (err) {
            console.error(`[Error]: Cannot create directory path for: ${this.hookPath}`);
            callback(err);
            return;
        }
        /* To support hook logic for existing node skills (backward compatibility) */
        if (this.runTime && this.runTime.startsWith('node')) {
            let scriptList = null;
            if (this.category === CONSTANTS.HOOKS.CATEGORIES.POST_NEW) {
                scriptList = [CONSTANTS.HOOKS.CATEGORIES.FILE_NAME.POST_NEW[this.osType], CONSTANTS.HOOKS.CATEGORIES.FILE_NAME.PRE_DEPLOY[this.osType]];
            } else {
                scriptList = [CONSTANTS.HOOKS.CATEGORIES.FILE_NAME.PRE_DEPLOY[this.osType]];
            }
            const scriptPath = url.resolve(CONSTANTS.HOOKS.TEMPLATES_URL, 'node/');
            this._cloneHookScripts(scriptPath, scriptList, (err) => {
                callback(err);
            });
        } else {
        // Creating an empty pre_deploy_hook for all skills which are not node runtime.
            const hookScript = path.join(this.hookPath, CONSTANTS.HOOKS.CATEGORIES.FILE_NAME.PRE_DEPLOY[this.osType]);
            fs.writeFileSync(hookScript, '', { mode: 0o755 });
            callback(null);
        }
    }

    /**
     * Executes the hook scripts
     * @param {function} callback
     */
    execute(callback) {
        let command;
        if (this.osType === 'WINDOWS') {
            const doDebug = this.doDebug ? '$True' : '$False';
            command = `PowerShell.exe -Command "& {& '${this.scriptPath}' ${this.skillName} ${doDebug} ${this.target}}"`;
        } else {
            command = `${this.scriptPath} ${this.skillName} ${this.doDebug} ${this.target}`;
        }
        shell.exec(command, { async: true }, (code, stdout, stderr) => {
            if (code !== 0) {
                callback('[Error]: Hook Scripts failed', null);
            } else if (stderr) {
                callback('[Error]: Hook Scripts failed', null);
            } else {
                callback(null, `Successful executed ${this.category} hook script`);
            }
        });
    }

    /**
     * Returns a source directory path if exists from the skill.json
     */
    _getSourceDir() {
        const skillFilePath = path.join(this.hookPath, '../skill.json');
        const manifest = jsonfile.readFileSync(skillFilePath);
        const apiList = jsonUtility.getPropertyValueFromObject(manifest, ['manifest', 'apis']);
        if (apiList) {
            for (const domain of Object.keys(apiList)) {
                const { endpoint } = apiList[domain];
                if (endpoint && endpoint.sourceDir) {
                    return endpoint.sourceDir;
                }
            }
        }
        return null;
    }

    /**
     * Extract runtime from ask/config file and also check for multiple language runtime
     */
    _getRunTime() {
        let runtime = null;
        const configPath = path.join(this.hookPath, '..', '.ask', 'config');
        const config = jsonUtility.read(configPath);
        const defaultConfig = jsonUtility.getPropertyValueFromObject(config, ['deploy_settings', this.profile, 'resources', 'lambda']);
        if (defaultConfig) {
            for (const lambda of defaultConfig) {
                if (!runtime && lambda.runtime) {
                    runtime = lambda.runtime.replace(/[^a-z]/gi, '');
                } else if (lambda.runtime && !lambda.runtime.startsWith(runtime)) {
                    console.warn('[Warn]: Different lambda runtime found. CLI currently does not support this functionality. '
                    + 'Developers should either work on their own hook scripts to build lambda for different language. '
                    + 'Or use only one languages for the skill.\n');
                    return null;
                }
            }
        }
        if (!runtime) {
            // check if package.json exists in template
            if (!this._getSourceDir()) {
                return null;
            }
            const packagePath = path.join(this.hookPath, '..', this._getSourceDir(), 'package.json');
            return fs.existsSync(packagePath) ? 'node' : 'python';
        }
        return runtime;
    }

    /**
     * Download hook scripts from the given url
     * @param {string} scriptUrl the url for get request
     * @param {function} callback
     */
    _getHookScripts(scriptUrl, callback) {
        const params = {
            url: scriptUrl,
            method: 'GET'
        };
        httpClient.request(params, 'getHookScripts', this.doDebug, (error, response) => {
            if (error || response.statusCode === null) {
                callback(error, null);
            } else {
                callback(null, response.body);
            }
        });
    }

    /**
     * Download hook scripts from S3 bucket to provide hook functionality for existing skills
     * @param {string} scriptPath the url for scripts
     * @param {list} scriptList   the list of scripts to download
     * @param {function} callback
     */
    _cloneHookScripts(scriptPath, scriptList, callback) {
        const installSpinner = new displayUtil.Spinner('Downloading the hooks scripts');
        installSpinner.start();
        async.forEach(scriptList, (script, getScriptCallback) => {
            const cloneDirLeaf = path.join(this.hookPath, script);
            const scriptUrl = url.resolve(scriptPath, script);
            this._getHookScripts(scriptUrl, (error, data) => {
                if (error) {
                    getScriptCallback(error);
                    return;
                }
                if (data) {
                    fs.writeFileSync(cloneDirLeaf, data, { mode: 0o755 });
                    getScriptCallback();
                }
            });
        }, (error) => {
            installSpinner.stop();
            callback(error);
        });
        installSpinner.stop();
    }
}

/**
 * Instantiate the hook object after successful validation or throw error.
 * @param {string} category   type of the hook
 * @param {string} skillName  the filtered skill name
 * @param {string} runTime    the lambda run time
 * @param {boolean} doDebug   the ASK CLI debug mode
 * @param {string} target     the target for deploy
 */
function createManager(category, skillName, runTime, profile, doDebug, target, callback) {
    // instantiate
    const hookManager = new HookManager(category, skillName, runTime, profile, doDebug, target);
    // create hooks if hooks not found
    if (!fs.existsSync(hookManager.hookPath)) {
        hookManager.createHook((err) => {
            callback(err, err ? null : hookManager);
        });
    } else {
        process.nextTick(() => {
            callback(null, hookManager);
        });
    }
}

module.exports = {
    HookManager,
    createManager
};
